@@ -1,9 +1,15 @@
 import os
 import sys
-import unittest
+
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
+
 import contextlib
 from subprocess import Popen, PIPE
 from tempfile import mkstemp
+
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -27,18 +33,6 @@ else:
                           os.path.join(ROOT_DIR, 'autopep8.py'),)
 
 
-def py27_and_above(func):
-    if sys.version_info[0] == 2 and sys.version_info[1] < 7:
-        func = None
-    return func
-
-
-def only_py2(func):
-    if sys.version_info[0] != 2:
-        func = None
-    return func
-
-
 class TestUtils(unittest.TestCase):
 
     def test_find_newline_only_cr(self):
@@ -61,11 +55,6 @@ class TestUtils(unittest.TestCase):
         source = ["print 1\r\n", "print 2\r", "print3\r\n"]
         self.assertEqual(autopep8.CRLF, autopep8.find_newline(source))
 
-    def test_shorten_comment(self):
-        self.assertEqual('# ' + 'x' * 77 + '\n',
-                         autopep8.shorten_comment('# ' + 'x' * 100 + '\n',
-                                                  '\n'))
-
     def test_detect_encoding(self):
         self.assertEqual(
             'utf-8',
@@ -94,6 +83,27 @@ class TestUtils(unittest.TestCase):
             'a b',
             autopep8.fix_whitespace('a\t  \t  b', offset=1, replacement=' '))
 
+    def test_break_multi_line(self):
+        self.assertEqual(
+            'foo_bar_zap_bing_bang_boom(\n    111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
+            autopep8.break_multi_line(
+                'foo_bar_zap_bing_bang_boom(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
+                newline='\n', indent_word='    '))
+
+    def test_break_multi_line_should_not_break_too_long_line(self):
+        self.assertEqual(
+            None,
+            autopep8.break_multi_line(
+                'foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom(333,\n',
+                newline='\n', indent_word='    '))
+
+    def test_break_multi_line_should_not_modify_comment(self):
+        self.assertEqual(
+            None,
+            autopep8.break_multi_line(
+                '# foo_bar_zap_bing_bang_boom(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
+                newline='\n', indent_word='    '))
+
 
 class TestFixPEP8Error(unittest.TestCase):
 
@@ -103,10 +113,10 @@ class TestFixPEP8Error(unittest.TestCase):
     def tearDown(self):
         os.remove(self.tempfile[1])
 
-    def _inner_setup(self, line, options=""):
+    def _inner_setup(self, line, options=()):
         with open(self.tempfile[1], 'w') as temp_file:
             temp_file.write(line)
-        opts, _ = autopep8.parse_args(options.split() + [self.tempfile[1]])
+        opts, _ = autopep8.parse_args(list(options) + [self.tempfile[1]])
         sio = StringIO()
         autopep8.fix_file(filename=self.tempfile[1],
                           opts=opts,
@@ -235,7 +245,7 @@ def foo():
          (2 , 1)
     1
     if True:
-           print('hello')  
+           print('hello')\t
     2
 """.lstrip()
         fixed = """
@@ -321,7 +331,7 @@ print "E128", ("under-",
         self.assertEqual(self.result, fixed)
 
     def test_e12_reindent_with_multiple_fixes(self):
-        line="""
+        line = """
 
 sql = 'update %s set %s %s' % (from_table,
                                ','.join(['%s=%s' % (col, col) for col in cols]),
@@ -330,15 +340,15 @@ sql = 'update %s set %s %s' % (from_table,
         fixed = """
 
 sql = 'update %s set %s %s' % (from_table,
-                               ','.join(
-                                   ['%s=%s' % (col, col) for col in cols]),
+                               ','.join(['%s=%s' % (col, col) for col in cols]
+                                        ),
                                where_clause)
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
     def test_e12_tricky(self):
-        line="""
+        line = """
 #: E126
 if (
     x == (
@@ -363,7 +373,126 @@ if (
     pass
 """
         self._inner_setup(line)
-        #self.assertEqual(self.result, fixed)
+        self.assertEqual(self.result, fixed)
+
+    def test_e12_large(self):
+        self.maxDiff = None
+
+        line = """
+class BogusController(controller.CementBaseController):
+    class Meta:
+        pass
+
+class BogusController2(controller.CementBaseController):
+    class Meta:
+        pass
+
+class BogusController3(controller.CementBaseController):
+    class Meta:
+        pass
+
+class BogusController4(controller.CementBaseController):
+    class Meta:
+        pass
+
+class TestBaseController(controller.CementBaseController):
+    class Meta:
+        pass
+
+class TestBaseController2(controller.CementBaseController):
+    class Meta:
+        pass
+
+class TestStackedController(controller.CementBaseController):
+    class Meta:
+        arguments = [
+            ]
+
+class TestDuplicateController(controller.CementBaseController):
+    class Meta:
+
+        config_defaults = dict(
+            foo='bar',
+            )
+
+        arguments = [
+            (['-f2', '--foo2'], dict(action='store'))
+            ]
+
+    def my_command(self):
+        pass
+"""
+        fixed = """
+
+class BogusController(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class BogusController2(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class BogusController3(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class BogusController4(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class TestBaseController(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class TestBaseController2(controller.CementBaseController):
+    class Meta:
+        pass
+
+
+class TestStackedController(controller.CementBaseController):
+    class Meta:
+        arguments = [
+        ]
+
+
+class TestDuplicateController(controller.CementBaseController):
+    class Meta:
+
+        config_defaults = dict(
+            foo='bar',
+        )
+
+        arguments = [
+            (['-f2', '--foo2'], dict(action='store'))
+        ]
+
+    def my_command(self):
+        pass
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e12_with_bad_indentation(self):
+        line = r"""
+
+
+def bar():
+    foo(1,
+      2)
+
+
+def baz():
+     pass
+
+    pass
+"""
+        self._inner_setup(line, options=['--select=E12'])
+        self.assertEqual(self.result, line)
 
     def test_e191(self):
         line = """
@@ -391,6 +520,19 @@ while True:
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e202_skip_multiline(self):
+        """We skip this since pep8 reports the error as being on line 1."""
+        line = """
+
+('''
+a
+b
+c
+''' )
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
     def test_e203_colon(self):
         line = "{4 : 3}\n"
         fixed = "{4: 3}\n"
@@ -406,13 +548,13 @@ while True:
     def test_e203_semicolon(self):
         line = "print(a, end=' ') ; nl = 0\n"
         fixed = "print(a, end=' '); nl = 0\n"
-        self._inner_setup(line, options='--select=E203')
+        self._inner_setup(line, options=['--select=E203'])
         self.assertEqual(self.result, fixed)
 
     def test_e203_with_newline(self):
         line = "print(a\n, end=' ')\n"
         fixed = "print(a, end=' ')\n"
-        self._inner_setup(line, options='--select=E203')
+        self._inner_setup(line, options=['--select=E203'])
         self.assertEqual(self.result, fixed)
 
     def test_e211(self):
@@ -456,7 +598,7 @@ class Foo():
 \tdef __init__(self):
 \t\tx = 1 + 3
 """.lstrip()
-        self._inner_setup(line, options='--ignore=W191')
+        self._inner_setup(line, options=['--ignore=W191'])
         self.assertEqual(self.result, fixed)
 
     def test_e224(self):
@@ -482,7 +624,7 @@ class Foo():
 \tdef __init__(self):
 \t\tx = 3
 """.lstrip()
-        self._inner_setup(line, options='--ignore=W191')
+        self._inner_setup(line, options=['--ignore=W191'])
         self.assertEqual(self.result, fixed)
 
     def test_e225(self):
@@ -495,12 +637,12 @@ class Foo():
         line = """
 class Foo(object):
   def bar(self):
-    return self.elephant!=None
+    return self.elephant is not None
 """.lstrip()
         fixed = """
 class Foo(object):
     def bar(self):
-        return self.elephant != None
+        return self.elephant is not None
 """.lstrip()
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
@@ -514,25 +656,25 @@ class Foo(object):
     def test_e241(self):
         line = "l = (1,  2)\n"
         fixed = "l = (1, 2)\n"
-        self._inner_setup(line, "--ignore W")
+        self._inner_setup(line, ['--ignore=W'])
         self.assertEqual(self.result, fixed)
 
     def test_e241_double(self):
         line = "l = (1,   2)\n"
         fixed = "l = (1, 2)\n"
-        self._inner_setup(line, "--ignore W")
+        self._inner_setup(line, ['--ignore=W'])
         self.assertEqual(self.result, fixed)
 
     def test_e242(self):
         line = "l = (1,\t2)\n"
         fixed = "l = (1, 2)\n"
-        self._inner_setup(line, "--ignore W")
+        self._inner_setup(line, ['--ignore=W'])
         self.assertEqual(self.result, fixed)
 
     def test_e242_double(self):
         line = "l = (1,\t\t2)\n"
         fixed = "l = (1, 2)\n"
-        self._inner_setup(line, "--ignore W")
+        self._inner_setup(line, ['--ignore=W'])
         self.assertEqual(self.result, fixed)
 
     def test_e251(self):
@@ -544,7 +686,7 @@ class Foo(object):
     def test_e251_with_escaped_newline(self):
         line = "1\n\n\ndef a(arg=\\\n1):\n    print(arg)\n"
         fixed = "1\n\n\ndef a(arg=1):\n    print(arg)\n"
-        self._inner_setup(line, options='--select=E251')
+        self._inner_setup(line, options=['--select=E251'])
         self.assertEqual(self.result, fixed)
 
     def test_e251_with_calling(self):
@@ -553,6 +695,12 @@ class Foo(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e251_with_argument_on_next_line(self):
+        line = 'foo(bar\n=None)\n'
+        fixed = 'foo(bar=None)\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e261(self):
         line = "print 'a b '# comment\n"
         fixed = "print 'a b '  # comment\n"
@@ -571,6 +719,12 @@ class Foo(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e261_with_comma(self):
+        line = "{1: 2 # comment\n , }\n"
+        fixed = "{1: 2  # comment\n , }\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e262_more_space(self):
         line = "print 'a b '  #  comment\n"
         fixed = "print 'a b '  # comment\n"
@@ -679,37 +833,75 @@ class Foo(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e401_should_ignore_commented_comma(self):
+        line = "import bdist_egg, egg  # , not a module, neither is this\n"
+        fixed = "import bdist_egg\nimport egg  # , not a module, neither is this\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e401_should_ignore_commented_comma_with_indentation(self):
+        line = "if True:\n    import bdist_egg, egg  # , not a module, neither is this\n"
+        fixed = "if True:\n    import bdist_egg\n    import egg  # , not a module, neither is this\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e401_should_ignore_false_positive(self):
+        line = "import bdist_egg; bdist_egg.write_safety_flag(cmd.egg_info, safe)\n"
+        self._inner_setup(line, options=['--select=E401'])
+        self.assertEqual(self.result, line)
+
     def test_e501_basic(self):
-        line = \
-"""print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 333, 333)
+        line = """
+
+print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 333, 333)
 """
-        fixed = \
-"""print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333,
+        fixed = """
+
+print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333,
       333, 333, 333)
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
     def test_e501_with_indent(self):
-        line = \
-"""def d():
+        line = """
+
+def d():
     print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 333, 333)
 """
-        fixed = \
-"""def d():
+        fixed = """
+
+def d():
     print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222,
           222, 333, 333, 333, 333)
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e501_alone_with_indentation(self):
+        line = """
+
+if True:
+    print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 333, 333)
+"""
+        fixed = """
+
+if True:
+    print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222,
+        222, 333, 333, 333, 333)
+"""
+        self._inner_setup(line, options=['--select=E501'])
+        self.assertEqual(self.result, fixed)
+
     def test_e501_arithmetic_operator_with_indent(self):
-        line = \
-"""def d():
+        line = """
+
+def d():
     111 + 111 + 111 + 111 + 111 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 333 + 333 + 333 + 333
 """
-        fixed = \
-"""def d():
+        fixed = """
+
+def d():
     111 + 111 + 111 + 111 + 111 + 222 + 222 + 222 + 222 + 222 + 222 + \\
         222 + 222 + 222 + 333 + 333 + 333 + 333
 """
@@ -717,19 +909,22 @@ class Foo(object):
         self.assertEqual(self.result, fixed)
 
     def test_e501_more_complicated(self):
-        line = \
-"""blahblah = os.environ.get('blahblah') or os.environ.get('blahblahblah') or os.environ.get('blahblahblahblah')
+        line = """
+
+blahblah = os.environ.get('blahblah') or os.environ.get('blahblahblah') or os.environ.get('blahblahblahblah')
 """
-        fixed = \
-"""blahblah = os.environ.get('blahblah') or os.environ.get(
+        fixed = """
+
+blahblah = os.environ.get('blahblah') or os.environ.get(
     'blahblahblah') or os.environ.get('blahblahblahblah')
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
     def test_e501_skip_even_more_complicated(self):
-        line = \
-"""if True:
+        line = """
+
+if True:
     if True:
         if True:
             blah = blah.blah_blah_blah_bla_bl(blahb.blah, blah.blah,
@@ -741,25 +936,44 @@ class Foo(object):
 
     def test_e501_prefer_to_break_at_begnning(self):
         """We prefer not to leave part of the arguments hanging."""
-        line = \
-"""looooooooooooooong = foo(one, two, three, four, five, six, seven, eight, nine, ten)
+        line = """
+
+looooooooooooooong = foo(one, two, three, four, five, six, seven, eight, nine, ten)
 """
-        fixed = \
-"""looooooooooooooong = foo(
+        fixed = """
+
+looooooooooooooong = foo(
     one, two, three, four, five, six, seven, eight, nine, ten)
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e501_with_multiple_lines(self):
+        line = """
+
+foo_bar_zap_bing_bang_boom(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,
+                           111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333)
+"""
+        fixed = """
+
+foo_bar_zap_bing_bang_boom(
+    111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,
+    111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333)
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e501_do_not_break_on_keyword(self):
         # We don't want to put a newline after equals for keywords as this
         # violates PEP 8.
-        line = \
-"""if True:
+        line = """
+
+if True:
     long_variable_name = tempfile.mkstemp(prefix='abcdefghijklmnopqrstuvwxyz0123456789')
 """
-        fixed = \
-"""if True:
+        fixed = """
+
+if True:
     long_variable_name = tempfile.mkstemp(
         prefix='abcdefghijklmnopqrstuvwxyz0123456789')
 """
@@ -784,56 +998,39 @@ def dummy():
     if True:
         if True:
             if True:
-                object = ModifyAction([MODIFY70.text, OBJECTBINDING71.text, COLON72.text],
-                                      MODIFY70.getLine(), MODIFY70.getCharPositionInLine())
+                object = ModifyAction([MODIFY70.text, OBJECTBINDING71.text, COLON72.text], MODIFY70.getLine(), MODIFY70.getCharPositionInLine())
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
     def test_e501_do_not_break_if_useless(self):
-        line = \
-"""
+        line = """
+
 123
 ('bbb', 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
 """
         self._inner_setup(line)
         self.assertEqual(self.result, line)
 
-    def test_e501_with_comment(self):
-        line = """123
-                        # This is a long comment that should be wrapped. I will wrap it using textwrap.
-"""
-        fixed = """123
-                        # This is a long comment that should be wrapped. I will
-                        # wrap it using textwrap.
-"""
-        self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
+    def test_e501_should_not_interfere_with_non_comment(self):
+        """This is why we've disabled comment shortening."""
+        line = '''
 
-    def test_e501_with_more_comments(self):
-        line = """123
-                        # This is a long comment that should be wrapped. I will wrap it using textwrap.
-                        # This is the next line.
-"""
-        fixed = """123
-                        # This is a long comment that should be wrapped. I will
-                        # wrap it using textwrap.
-                        # This is the next line.
 """
+# not actually a comment %d. 12345678901234567890, 12345678901234567890, 12345678901234567890.
+""" % (0,)
+'''
         self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
+        self.assertEqual(self.result, line)
 
-    def test_e501_should_cut_comment_pattern(self):
-        line = """123
-# -- Randy's datastructure -----------------------------------------------------
-321
-"""
-        fixed = """123
-# -- Randy's datastructure ----------------------------------------------------
-321
-"""
+    def test_e501_should_not_break_on_dot(self):
+        line = """
+if True:
+    if True:
+        raise xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx('xxxxxxxxxxxxxxxxx "{d}" xxxxxxxxxxxxxx'.format(d='xxxxxxxxxxxxxxx'))
+""".lstrip()
         self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
+        self.assertEqual(self.result, line)
 
     def test_e502(self):
         line = "print('abc'\\\n      'def')\n"
@@ -853,6 +1050,18 @@ def dummy():
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_with_escaped_newline(self):
+        line = '1; \\\n2\n'
+        fixed = '1\n2\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e702_with_escaped_newline_with_indentation(self):
+        line = '1; \\\n    2\n'
+        fixed = '1\n2\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e702_more_complicated(self):
         line = """\
 def foo():
@@ -901,11 +1110,15 @@ def foo():
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    def test_e702_skip_with_triple_quote(self):
-        # We do not support this yet.
-        # We would expect '"""\n      hello\n   """\n1\n'.
+    def test_e702_with_triple_quote(self):
         line = '"""\n      hello\n   """; 1\n'
-        fixed = line
+        fixed = '"""\n      hello\n   """\n1\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e702_with_triple_quote_and_indent(self):
+        line = '    """\n      hello\n   """; 1\n'
+        fixed = '    """\n      hello\n   """\n    1\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
@@ -921,6 +1134,48 @@ raise IOError('abc '
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e711(self):
+        line = 'foo == None\n'
+        fixed = 'foo is None\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e711_in_conditional(self):
+        line = 'if foo == None and None == foo:\npass\n'
+        fixed = 'if foo is None and None == foo:\npass\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e711_in_conditional_with_multiple_instances(self):
+        line = 'if foo == None and bar == None:\npass\n'
+        fixed = 'if foo is None and bar is None:\npass\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e711_with_not_equals_none(self):
+        line = 'foo != None\n'
+        fixed = 'foo is not None\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e721(self):
+        line = "type('') == type('')\n"
+        fixed = "isinstance('', type(''))\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e721_with_str(self):
+        line = "str == type('')\n"
+        fixed = "isinstance('', str)\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e721_in_conditional(self):
+        line = "if str == type(''):\n    pass\n"
+        fixed = "if isinstance('', str):\n    pass\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
 
 class TestFixPEP8Warning(unittest.TestCase):
 
@@ -1006,151 +1261,140 @@ class TestFixPEP8Warning(unittest.TestCase):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    def test_w601_skip_multiple(self):
-        # We don't support this case
+    def test_w601_with_multiple(self):
         line = "a.has_key(0) and b.has_key(0)\n"
-        self._inner_setup(line)
-        self.assertEqual(self.result, line)
-
-    def test_w601_skip_multiple_nested(self):
-        # We don't support this case
-        line = "alpha.has_key(nested.has_key(12)) and beta.has_key(1)\n"
-        self._inner_setup(line)
-        self.assertEqual(self.result, line)
-
-    def test_w601_precedence(self):
-        line = "if self.a.has_key(1 + 2):\n    print 1\n"
-        fixed = "if (1 + 2) in self.a:\n    print 1\n"
+        fixed = "0 in a and 0 in b\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    def test_w601_skip_parens(self):
-        # We don't support this case
-        line = "alpha.has_key(foo(12))\n"
+    def test_w601_with_multiple_nested(self):
+        line = "alpha.has_key(nested.has_key(12)) and beta.has_key(1)\n"
+        fixed = "(12 in nested) in alpha and 1 in beta\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w601_with_more_complexity(self):
+        line = 'y.has_key(0) + x.has_key(x.has_key(0) + x.has_key(x.has_key(0) + x.has_key(1)))\n'
+        fixed = '(0 in y) + ((0 in x) + ((0 in x) + (1 in x) in x) in x)\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w601_precedence(self):
+        line = "if self.a.has_key(1 + 2):\n    print 1\n"
+        fixed = "if 1 + 2 in self.a:\n    print 1\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w601_with_parens(self):
+        line = "foo(12) in alpha\n"
         self._inner_setup(line)
         self.assertEqual(self.result, line)
 
-    def test_w601_skip_multi_line(self):
-        # We don't support this case
+    def test_w601_with_multi_line(self):
         line = """
+
 a.has_key(
     0
 )
 """.lstrip()
-        fixed = line
+        fixed = '0 in a\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_arg_is_string(self):
         line = "raise ValueError, \"w602 test\"\n"
         fixed = "raise ValueError(\"w602 test\")\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_arg_is_string_with_comment(self):
         line = "raise ValueError, \"w602 test\"  # comment\n"
         fixed = "raise ValueError(\"w602 test\")  # comment\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_skip_ambiguous_case(self):
         line = "raise 'a', 'b', 'c'\n"
         self._inner_setup(line)
         self.assertEqual(self.result, line)
 
-    @only_py2
-    def test_w602_skip_logic(self):
+    def test_w602_with_logic(self):
         line = "raise TypeError, e or 'hello'\n"
+        fixed = "raise TypeError(e or 'hello')\n"
         self._inner_setup(line)
-        self.assertEqual(self.result, line)
+        self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_triple_quotes(self):
         line = 'raise ValueError, """hello"""\n1\n'
         fixed = 'raise ValueError("""hello""")\n1\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline(self):
         line = 'raise ValueError, """\nhello"""\n'
         fixed = 'raise ValueError("""\nhello""")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
-    def test_w602_skip_complex_multiline(self):
-        # We do not handle formatted multiline strings
+    def test_w602_with_complex_multiline(self):
         line = 'raise ValueError, """\nhello %s %s""" % (\n    1, 2)\n'
+        fixed = 'raise ValueError("""\nhello %s %s""" % (\n    1, 2))\n'
         self._inner_setup(line)
-        self.assertEqual(self.result, line)
+        self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_with_trailing_spaces(self):
         line = 'raise ValueError, """\nhello"""    \n'
         fixed = 'raise ValueError("""\nhello""")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_with_escaped_newline(self):
         line = 'raise ValueError, \\\n"""\nhello"""\n'
         fixed = 'raise ValueError("""\nhello""")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_with_escaped_newline_and_comment(self):
         line = 'raise ValueError, \\\n"""\nhello"""  # comment\n'
         fixed = 'raise ValueError("""\nhello""")  # comment\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_with_multiple_escaped_newlines(self):
         line = 'raise ValueError, \\\n\\\n\\\n"""\nhello"""\n'
         fixed = 'raise ValueError("""\nhello""")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_with_nested_quotes(self):
         line = 'raise ValueError, """hello\'\'\'blah"a"b"c"""\n'
         fixed = 'raise ValueError("""hello\'\'\'blah"a"b"c""")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
-    def test_w602_skip_multiline_with_single_quotes(self):
+    def test_w602_with_multiline_with_single_quotes(self):
         line = "raise ValueError, '''\nhello'''\n"
         fixed = "raise ValueError('''\nhello''')\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiline_string_stays_the_same(self):
         line = 'raise """\nhello"""\n'
         self._inner_setup(line)
         self.assertEqual(self.result, line)
 
-    @only_py2
     def test_w602_escaped_lf(self):
         line = 'raise ValueError, \\\n"hello"\n'
         fixed = 'raise ValueError("hello")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_escaped_crlf(self):
         line = 'raise ValueError, \\\r\n"hello"\n'
         fixed = 'raise ValueError("hello")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_indentation(self):
         line = 'def foo():\n    raise ValueError, "hello"\n'
         fixed = 'def foo():\n    raise ValueError("hello")\n'
@@ -1164,42 +1408,45 @@ a.has_key(
     #    self._inner_setup(line)
     #    self.assertEqual(self.result, fixed)
 
-    @only_py2
     def test_w602_multiple_statements(self):
         line = 'raise ValueError, "hello";print 1\n'
         fixed = 'raise ValueError("hello")\nprint 1\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    @only_py2
-    def test_w602_raise_argument_triple(self):
+    def test_w602_raise_argument_with_indentation(self):
+        line = 'if True:\n    raise ValueError, "error"\n'
+        fixed = 'if True:\n    raise ValueError("error")\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w602_skip_raise_argument_triple(self):
         line = 'raise ValueError, "info", traceback\n'
-        fixed = 'raise ValueError("info"), None, traceback\n'
         self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
+        self.assertEqual(self.result, line)
 
-    @only_py2
-    def test_w602_raise_argument_triple_skip_with_indentation(self):
-        # We do not handle this properly yet.
-        line = 'if True:\n    raise TypeError, "error", tb\n'
-        fixed = line
-        self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
-
-    @only_py2
-    def test_w602_raise_argument_triple_with_comment(self):
+    def test_w602_skip_raise_argument_triple_with_comment(self):
         line = 'raise ValueError, "info", traceback  # comment\n'
-        fixed = 'raise ValueError("info"), None, traceback  # comment\n'
         self._inner_setup(line)
-        self.assertEqual(self.result, fixed)
+        self.assertEqual(self.result, line)
 
-    @only_py2
     def test_w602_raise_argument_triple_fake(self):
         line = 'raise ValueError, "info, info2"\n'
         fixed = 'raise ValueError("info, info2")\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_w602_with_list_comprehension(self):
+        line = "raise Error, [x[0] for x in probs]\n"
+        fixed = "raise Error([x[0] for x in probs])\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w602_with_bad_syntax(self):
+        line = "raise Error, 'abc\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
     def test_w603(self):
         line = "if 2 <> 2:\n    print False"
         fixed = "if 2 != 2:\n    print False\n"
@@ -1212,17 +1459,15 @@ a.has_key(
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    def test_w604_skip_multiple_instances(self):
-        # We do not support this yet
-        line = "``1`` + ``b``\n"
-        fixed = line
+    def test_w604_with_multiple_instances(self):
+        line = '``1`` + ``b``\n'
+        fixed = 'repr(repr(1)) + repr(repr(b))\n'
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
-    def test_w604_skip_multiple_lines(self):
-        # We do not support this yet
-        line = "`(1\n  )`\n"
-        fixed = line
+    def test_w604_with_multiple_lines(self):
+        line = "`(1\n      )`\n"
+        fixed = "repr((1\n      ))\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
@@ -1259,13 +1504,11 @@ class TestOptions(unittest.TestCase):
         self._inner_setup(line, ['--ignore=E,W'])
         self.assertEqual(self.result, line)
 
-    @py27_and_above
     def test_help(self):
         p = Popen(list(AUTOPEP8_CMD_TUPLE) + ['-h'],
                   stdout=PIPE)
         self.assertIn('Usage:', p.communicate()[0].decode('utf8'))
 
-    @py27_and_above
     def test_verbose(self):
         line = 'bad_syntax)'
         f = open(self.tempfile[1], 'w')
@@ -1276,7 +1519,6 @@ class TestOptions(unittest.TestCase):
         verbose_error = p.communicate()[1].decode('utf8')
         self.assertIn("'fix_e901' is not defined", verbose_error)
 
-    @py27_and_above
     def test_verbose_diff(self):
         line = 'bad_syntax)'
         f = open(self.tempfile[1], 'w')
@@ -1304,14 +1546,13 @@ class TestOptions(unittest.TestCase):
 
     def test_in_place_and_diff(self):
         line = "'abc'  \n"
-        fixed = "'abc'\n"
         f = open(self.tempfile[1], 'w')
         f.write(line)
         f.close()
         p = Popen(list(AUTOPEP8_CMD_TUPLE) + [self.tempfile[1], '--in-place', '--diff'],
                   stderr=PIPE)
         result = p.communicate()[1].decode('utf8')
-        self.assertTrue('--in-place and --diff are mutually exclusive' in result)
+        self.assertIn('--in-place and --diff are mutually exclusive', result)
 
     def test_recursive(self):
         import tempfile
@@ -1343,14 +1584,13 @@ class TestOptions(unittest.TestCase):
 
     def test_only_recursive(self):
         line = "'abc'  \n"
-        fixed = "'abc'\n"
         f = open(self.tempfile[1], 'w')
         f.write(line)
         f.close()
         p = Popen(list(AUTOPEP8_CMD_TUPLE) + [self.tempfile[1], '--recursive'],
                   stderr=PIPE)
         result = p.communicate()[1].decode('utf8')
-        self.assertTrue('must be used with --in-place or --diff' in result)
+        self.assertIn('must be used with --in-place or --diff', result)
 
 
 class TestSpawnPEP8Process(unittest.TestCase):
@@ -1361,10 +1601,10 @@ class TestSpawnPEP8Process(unittest.TestCase):
     def tearDown(self):
         os.remove(self.tempfile[1])
 
-    def _inner_setup(self, line, options=""):
+    def _inner_setup(self, line, options=()):
         with open(self.tempfile[1], 'w') as temp_file:
             temp_file.write(line)
-        opts, _ = autopep8.parse_args(options.split() + [self.tempfile[1]])
+        opts, _ = autopep8.parse_args(list(options) + [self.tempfile[1]])
         sio = StringIO()
 
         # Monkey patch pep8 to trigger spawning
@@ -1385,6 +1625,15 @@ class TestSpawnPEP8Process(unittest.TestCase):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_verbose(self):
+        line = "print('abc' )    \n1*1\n"
+        fixed = "print('abc')\n1 * 1\n"
+        sio = StringIO()
+        with capture_stderr(sio):
+            self._inner_setup(line, options=['--verbose'])
+        self.assertEqual(self.result, fixed)
+        self.assertIn('compatibility mode', sio.getvalue())
+
 
 class TestCoverage(unittest.TestCase):
 
@@ -1394,7 +1643,7 @@ class TestCoverage(unittest.TestCase):
     def tearDown(self):
         os.remove(self.tempfile[1])
 
-    def _inner_setup(self, line, options=""):
+    def _inner_setup(self, line):
         with open(self.tempfile[1], 'w') as temp_file:
             temp_file.write(line)
 
@@ -1423,13 +1672,19 @@ class TestCoverage(unittest.TestCase):
 
 @contextlib.contextmanager
 def disable_stderr():
-    with open('/dev/null', 'w') as fake_stderr:
-        _tmp = sys.stderr
-        sys.stderr = fake_stderr
-        try:
-            yield
-        finally:
-            sys.stderr = _tmp
+    sio = StringIO()
+    with capture_stderr(sio):
+        yield
+
+
+@contextlib.contextmanager
+def capture_stderr(sio):
+    _tmp = sys.stderr
+    sys.stderr = sio
+    try:
+        yield
+    finally:
+        sys.stderr = _tmp
 
 
 if __name__ == '__main__':
